module Condor
  class Pool
    attr_reader :jobs
    attr_reader :waiting

    def initialize
      @jobs = []
      @waiting = {}
    end

    # run command, given job dependencies
    def add(cmd, dep = nil, priority = nil, dir = nil)
      return if !cmd or cmd.empty?
      c = Job.new(cmd, :priority => priority, :directory => dir)
      if [dep].flatten.compact.empty?
        c.run
      else
        @waiting[c] = [dep].flatten.compact
      end
      @jobs << c
      update
      return c
    end

    def update
      Condor::Queue.load(true)
      @jobs.each do |j|
        j.update
        next if !(deps = @waiting[j])
        if deps.all?(&:done?)
          j.run
          @waiting.delete(j)
        end
      end
    end

    def finalize
      while alive?
        sleep 2
        update
      end
      @jobs.each{|x| x.cleanup}
      @jobs = []
    end

    def alive?
      return true if !@waiting.empty?
      @jobs.any?(&:done?)
    end
  end
end
